--[[

CombatCaddy

A simple addon that does a sequence of events when a player enters
and/or exits combat. 

Features:
 - Hides minimap when entering combat.
 - Displays minimap when player is at low health.
 - Hides armor durability when entering combat.
 - Hides buffs/debuffs when entering combat.
 - Hides current chat frame when entering combat.
 - Closes the world map when entering combat.
 - Closes open bags when entering combat.
 - Switches to combat log when entering combat.
 - Allows user to specify search metric for the combat log chat window.
 - Plays a mp3 during combat.
 - Says a random battlecry when entering combat.
 - Listens for combat from party/raid members.
 
Changes:
0.2 -  Should take into account pre-combat UI settings and restore them when leaving combat.
       Added setup screen with saved options.
       Fixed a minimap hide bug that magically appeared by calling Minimap:SetAlpha(0) instead
       of Minimap:Hide().
0.3 -  Added more features and options.
       Addon now updates the OPTIONS list as necessary.
0.4 -  Fixed event handler to handle switching to the combat log correctly.
       Added more features and options.
       Added hooks for option dependencies.
0.5 -  Fixed the interaction between working options and saved options.
       Handles the escape key being pressed on the main frame like cancel button hit.
       Changed option input dialog so current value is displayed in edit box.
0.6 -  Added more features and options.
       Fixed option input dialog button placement bug.
0.7 -  Fixed options not disabling bug.
       Added mp3 track generation list script and hooks for it.
       Added random mp3 option.
       Added ability to unlink combat music volume from game music volume.
0.8 -  Implemented party/raid watch.
       Modularizing some things to make the addon cleaner.
       Made the version come from a single source.
       Changed from VARIABLES_LOADED to ADDON_LOADED.
       Minor aesthetic changes.
       Added auto-disable by zone type options.
0.9 -  Implemented multiple battlecry entry with random selection.
       Options are now saved per character instead of per account.
       Moved input dialog junk to its own files.
       Low health threshold now supports percentages.
0.10 - Made sure the battlecry is not the same twice in a row.
       Modified the minimap hiding code.

]]--

-- Do not overwrite stored variables
CombatCaddy = {};
WORKING_OPTIONS = {};

CombatCaddy.title = "|cff1ca01cCombat|r|cffa06020Caddy|r";

-- Processing variables
CombatCaddy.enabled = true;
CombatCaddy.manualEnabled = true;
CombatCaddy.incombat = false;
CombatCaddy.combatLogIdx = nil;
CombatCaddy.curOptName = nil;
CombatCaddy.curOptKey = nil;
CombatCaddy.fadeMusic = false;
CombatCaddy.startMusic = false;
CombatCaddy.musicVol = nil;
CombatCaddy.combatants = {};
CombatCaddy.checkCombat = false;
CombatCaddy.zone = nil;
CombatCaddy.subzone = nil;

-- Pre-combat UI element state vars
CombatCaddy.mmState = true;
CombatCaddy.chatState = true;
CombatCaddy.chatIdx = 1;
CombatCaddy.durState = 0;
CombatCaddy.qwatchState = 0;
CombatCaddy.gameMusicVol = nil;

-- Init functions
CombatCaddy.OnLoad = function()
	-- Set up slash commands
	SlashCmdList["CMBCADCOMMAND"] = CombatCaddy.Cmd;
	SLASH_CMBCADCOMMAND1 = "/cmbcdy";
	SLASH_CMBCADCOMMAND2 = "/combatcaddy";
	
	-- Register events
	this:RegisterEvent("ADDON_LOADED");
	this:RegisterEvent("PLAYER_REGEN_ENABLED");
	this:RegisterEvent("PLAYER_REGEN_DISABLED");
	this:RegisterEvent("UNIT_COMBAT");
	this:RegisterEvent("ON_UPDATE");
	this:RegisterEvent("CHAT_MSG_ADDON");
	this:RegisterEvent("ZONE_CHANGED");
	this:RegisterEvent("ZONE_CHANGED_NEW_AREA");

	this:SetScript("OnEvent", CombatCaddy.OnEvent);
	this:SetScript("OnUpdate", CombatCaddy.OnUpdate);

	CombatCaddy.zone = GetSubZoneText();
	CombatCaddy.subzone = GetRealZoneText();
end

-- Event Handler
CombatCaddy.OnEvent = function(self, event, ...)
	if (event == "ADDON_LOADED") then
		if (arg1 ~= "CombatCaddy") then return; end

		-- Get the CombatCaddy version info
		local toc_ver = GetAddOnMetadata("CombatCaddy","Version");
		if (not toc_ver) then CombatCaddy.VERSION = "unknown";
		else CombatCaddy.VERSION = toc_ver; end
	
		-- Initialize saved data (if necessary)
		if (not CombatCaddyOptions) then
			CombatCaddyOptions = {};
		end

		-- Update the options list (if necessary)
		for name, opt in pairs(CombatCaddy.DEFAULT_OPTIONS) do
			if (not CombatCaddyOptions[name]) then
				CombatCaddyOptions[name] = opt;
			else
				-- Fill in nil option fields with defaults
				for k,v in pairs(opt) do
					if ((k ~= "enabled") and (not CombatCaddyOptions[name][k])) then
						CombatCaddyOptions[name][k] = v;
					end
				end

				-- Normalize enabled logic
				if (not CombatCaddyOptions[name].enabled or CombatCaddyOptions[name].enabled == 0) then
					CombatCaddyOptions[name].enabled = false;
				else
					CombatCaddyOptions[name].enabled = true;
				end

				-- Update option text
				CombatCaddyOptions[name].text = opt.text;
				
				-- Update deps
				if (opt.enables) then
					CombatCaddyOptions[name].enables = opt.enables;
				end
				if (opt.disables) then
					CombatCaddyOptions[name].disables = opt.disables;
				end
			end
		end

		CombatCaddy.OptionsSetup();

		for name,_ in pairs(CombatCaddyOptions) do
			WORKING_OPTIONS[name] = {};
			for k,v in pairs(CombatCaddyOptions[name]) do
				WORKING_OPTIONS[name][k] = v;
			end
		end

		for name,_ in pairs(WORKING_OPTIONS) do
			CombatCaddy.OptionsDeps(name);
		end

		CombatCaddy.Minimap.Update();

		DEFAULT_CHAT_FRAME:AddMessage(CombatCaddy.title.." v. "..CombatCaddy.VERSION.." loaded, try /cmbcdy or /combatcaddy.", 1.0, 1.0, 0.0);
	elseif (event == "ZONE_CHANGED") then
		CombatCaddy.subzone = GetSubZoneText();
		CombatCaddy.AutoDisable();
	elseif (event == "ZONE_CHANGED_NEW_AREA") then
		CombatCaddy.zone = GetRealZoneText();
		CombatCaddy.AutoDisable();
	end

	-- If CombatCaddy is "disabled," drop out
	if (not CombatCaddy.enabled) then return; end

	if (event == "PLAYER_REGEN_DISABLED") then
		-- Annouce to party / raid
		if (UnitInRaid("player")) then
			SendAddonMessage("CombatCaddy", "ENTERING_COMBAT:"..tostring(CombatCaddy.zone)..":"..tostring(CombatCaddy.subzone), "RAID");
		end
		if (UnitInParty("player")) then
			SendAddonMessage("CombatCaddy", "ENTERING_COMBAT:"..tostring(CombatCaddy.zone)..":"..tostring(CombatCaddy.subzone), "PARTY");
		end

		if (not CombatCaddy.incombat) then
			CombatCaddy.incombat = true;
			CombatCaddy.combatants[UnitName("player")] = true;

			CombatCaddy.DoCombatActivities();
		end
	elseif (event == "PLAYER_REGEN_ENABLED") then
		CombatCaddy.incombat = false;
		CombatCaddy.combatants[UnitName("player")] = nil;
		CombatCaddy.checkCombat = true;

		-- Annouce to party / raid
		if (UnitInRaid("player")) then
			SendAddonMessage("CombatCaddy", "LEAVING_COMBAT", "RAID");
		end
		if (UnitInParty("player")) then
			SendAddonMessage("CombatCaddy", "LEAVING_COMBAT", "PARTY");
		end
	elseif (event == "UNIT_COMBAT") then
		if (CombatCaddyOptions["CheckLowHealthMinimap"].enabled) then
			local health = UnitHealth("player");
			local thresh = tonumber(CombatCaddyOptions["CheckLowHealthMinimap"].thresh);

			if (not thresh) then return; end --catch bad user input

			if (health <= thresh) then
				CombatCaddy.HideMinimap(false);
			end	
		end
	elseif (event == "CHAT_MSG_ADDON") then
		local prefix = arg1;
		local cmd = arg2;
		local chan = arg3;
		local from = arg4;
		local c;
		local action, zone, subzone;

		if (prefix == "CombatCaddy") then
			if (CombatCaddyOptions["CheckPartyWatch"].enabled or CombatCaddyOptions["CheckRaidWatch"].enabled) then
				c = strfind(cmd,":");
				if (not c) then
					action = cmd;
				else
					action = strsub(cmd,1,c-1);
					cmd = strsub(cmd,c+1,strlen(cmd));
				end

				if (action == "ENTERING_COMBAT") then
					-- Scrub out messages we aren't listening for
					if (
						(CombatCaddyOptions["CheckRaidWatch"].enabled and (chan ~= "RAID"))
							and
						(CombatCaddyOptions["CheckPartyWatch"].enabled and (chan ~= "PARTY"))
					) then return; end

					if (
						(CombatCaddyOptions["CheckRaidWatchLocal"].enabled and (chan == "RAID"))
							or
						(CombatCaddyOptions["CheckPartyWatchLocal"].enabled and (chan == "PARTY"))
					) then
						-- Parse out location information
						c = strfind(cmd, ":");
						zone = strsub(cmd,1,c-1);
						cmd = strsub(cmd,c+1,strlen(cmd));
						c = strfind(cmd, ":");
						subzone = strsub(cmd,1,strlen(cmd));

						if ((zone ~= CombatCaddy.zone) or (subzone ~= CombatCaddy.subzone)) then
							return; -- player is not local
						end
					end

					-- Add player to combat list
					CombatCaddy.combatants[from] = true;
					CombatCaddy.checkCombat = true;

					-- Announce first combat to party/raid
					if (not CombatCaddy.incombat) then
						if (CombatCaddyOptions["CheckRaidWatchWarn"].enabled and IsRaidLeader("player")) then
							SendChatMessage(from.." has entered combat!", "RAID_WARNING");
						elseif (CombatCaddyOptions["CheckPartyWatchWarn"].enabled and IsPartyLeader("player")) then
							SendChatMessage(from.." has entered combat!", "RAID_WARNING");
						end
					end

					if (
						(CombatCaddyOptions["CheckRaidWatchDoStuff"].enabled and (chan == "RAID")) 
							or
						(CombatCaddyOptions["CheckPartyWatchDoStuff"].enabled and (chan == "PARTY"))
					) then
						if (not CombatCaddy.incombat) then
							CombatCaddy.incombat = true;
							CombatCaddy.DoCombatActivities();
						end
					end
				elseif (action == "LEAVING_COMBAT") then
					-- Remove player from combat list
					if (not CombatCaddy.combatants[from]) then return; end -- nothing to do

					CombatCaddy.combatants[from] = nil;
					CombatCaddy.checkCombat = true;
				end
			end
		end
	end
end

CombatCaddy.AutoDisable = function()
	_,intype = IsInInstance();

	if ((intype == "none") and CombatCaddyOptions["CheckDisableInWorld"].enabled) then
		CombatCaddy.enabled = false;
	elseif ((intype == "pvp") and CombatCaddyOptions["CheckDisableInBG"].enabled) then
		CombatCaddy.enabled = false;
	elseif ((intype == "arena") and CombatCaddyOptions["CheckDisableInArena"].enabled) then
		CombatCaddy.enabled = false;
	elseif ((intype == "party") and CombatCaddyOptions["CheckDisableInPartyInstance"].enabled) then
		CombatCaddy.enabled = false;
	elseif ((intype == "raid") and CombatCaddyOptions["CheckDisableInRaidInstance"].enabled) then
		CombatCaddy.enabled = false;
	else
		CombatCaddy.enabled = CombatCaddy.manualEnabled;
	end
end

-- Container function for all entering combat activities
CombatCaddy.DoCombatActivities = function()
	CombatCaddy.CloseSelf();
	if (CombatCaddyOptions["CheckCloseAll"].enabled) then
		CombatCaddy.CloseAchievements();
		CombatCaddy.CloseBags();
		CombatCaddy.CloseChar();
		CombatCaddy.CloseLFG();
		CombatCaddy.ClosePVP();
		CombatCaddy.CloseQuestLog();
		CombatCaddy.CloseSocial();
		CombatCaddy.CloseSpellbook();
		CombatCaddy.CloseWorldmap();
	else 
		if (CombatCaddyOptions["CheckCloseAchievements"].enabled) then CombatCaddy.CloseAchievements(); end
		if (CombatCaddyOptions["CheckCloseBags"].enabled) then CombatCaddy.CloseBags(); end
		if (CombatCaddyOptions["CheckCloseChar"].enabled) then CombatCaddy.CloseChar(); end
		if (CombatCaddyOptions["CheckCloseLFG"].enabled) then CombatCaddy.CloseLFG(); end
		if (CombatCaddyOptions["CheckClosePVP"].enabled) then CombatCaddy.ClosePVP(); end
		if (CombatCaddyOptions["CheckCloseQuestLog"].enabled) then CombatCaddy.CloseQuestLog(); end
		if (CombatCaddyOptions["CheckCloseSocial"].enabled) then CombatCaddy.CloseSocial(); end
		if (CombatCaddyOptions["CheckCloseSpellbook"].enabled) then CombatCaddy.CloseSpellbook(); end
		if (CombatCaddyOptions["CheckCloseWorldmap"].enabled) then CombatCaddy.CloseWorldmap(); end
	end

	if (CombatCaddyOptions["CheckHideBuffs"].enabled) then CombatCaddy.HideBuffs(true); end
	if (CombatCaddyOptions["CheckHideDurability"].enabled) then CombatCaddy.HideDurability(true); end
	if (CombatCaddyOptions["CheckHideMinimap"].enabled) then CombatCaddy.HideMinimap(true); end
	if (CombatCaddyOptions["CheckHideQuestTracker"].enabled) then CombatCaddy.HideQuestTracker(true); end

	if (CombatCaddyOptions["CheckHideChat"].enabled) then 
		CombatCaddy.HideChatFrame(true); 
	elseif (CombatCaddyOptions["CheckSwitchToCombatLog"].enabled) then 
		CombatCaddy.SwitchToCombatLog(true); 
	end
	if (CombatCaddyOptions["CheckBattlecry"].enabled) then CombatCaddy.Battlecry(); end

	if (CombatCaddyOptions["CheckFightMusic"].enabled) then CombatCaddy.PlayMusic(); end
end

-- Container function for all exiting combat activities
CombatCaddy.DoPostCombatActivities = function()
	CombatCaddy.SwitchToCombatLog(false);
	CombatCaddy.HideBuffs(false);
	CombatCaddy.HideDurability(false);
	CombatCaddy.HideMinimap(false);
	CombatCaddy.HideQuestTracker(false);
	CombatCaddy.HideChatFrame(false);

	if (CombatCaddyOptions["CheckFightMusic"].enabled) then CombatCaddy.fadeMusic = true; end
end

-- Periodic function that handles forced combat and music fading
CombatCaddy.OnUpdate = function(self, ...)
		if (CombatCaddy.checkCombat) then
			CombatCaddy.checkCombat = false;
			
			isnonempty = next(CombatCaddy.combatants, nil);
			if (not isnonempty) then
				CombatCaddy.DoPostCombatActivities();
				CombatCaddy.incombat = false;
			end
		end

		-- Fade out combat music
		if (CombatCaddy.fadeMusic) then
			CombatCaddy.musicVol = CombatCaddy.musicVol - 0.01;
			if (CombatCaddy.musicVol <= 0) then
				StopMusic();
				PlayMusic("Interface\\AddOns\\CombatCaddy\\EmptyTrack.mp3");
				CombatCaddy.startMusic = true;	
				CombatCaddy.fadeMusic = false;
			else
				SetCVar("Sound_MusicVolume", CombatCaddy.musicVol);
			end
		elseif (CombatCaddy.startMusic) then
			CombatCaddy.musicVol = CombatCaddy.musicVol + 0.01;
			if (CombatCaddy.musicVol >= CombatCaddy.gameMusicVol) then
				StopMusic();
				CombatCaddy.startMusic = false;
				SetCVar("Sound_MusicVolume", CombatCaddy.gameMusicVol);
			else
				SetCVar("Sound_MusicVolume", CombatCaddy.musicVol);
			end
		end
end

-- Slash command parsing and processing
CombatCaddy.Cmd = function(msg)
	msg = strlower(msg);
	if ((msg == "help") or (msg == "?") or not msg) then
		DEFAULT_CHAT_FRAME:AddMessage(CombatCaddy.title.." v. "..CombatCaddy.VERSION, 1.0, 1.0, 0.0);
		DEFAULT_CHAT_FRAME:AddMessage("Usage: /cmbcdy [OPTIONS...] or /combatcaddy [OPTIONS...]", 1.0, 1.0, 0.0);
		DEFAULT_CHAT_FRAME:AddMessage("   enable  - enables CombatCaddy", 1.0, 1.0, 0.0);
		DEFAULT_CHAT_FRAME:AddMessage("   disable - disables CombatCaddy", 1.0, 1.0, 0.0);
		DEFAULT_CHAT_FRAME:AddMessage("   setup   - displays CombatCaddy setup dialog", 1.0, 1.0, 0.0);
		DEFAULT_CHAT_FRAME:AddMessage("   status  - displays enabled/disabled status of CombatCaddy", 1.0, 1.0, 0.0);
		DEFAULT_CHAT_FRAME:AddMessage("   help, ? - displays usage informatio", 1.0, 1.0, 0.0);
	elseif (msg == "") then
		CombatCaddy_Main:Show();
	elseif ((msg == "enable") or (msg == "ena")) then
		if (CombatCaddy.enabled ~= true) then
			CombatCaddy.manualEnabled = true;
			CombatCaddy.enabled = true;
			DEFAULT_CHAT_FRAME:AddMessage(CombatCaddy.title.." |cff00ff00enabled|r!", 1.0, 1.0, 0.0);
		end
	elseif ((msg == "disable") or (msg == "dis")) then
		if (CombatCaddy.enabled ~= false) then
			CombatCaddy.manualEnabled = false;
			CombatCaddy.enabled = false;
			DEFAULT_CHAT_FRAME:AddMessage(CombatCaddy.title.." |cffff0000disabled|r!", 1.0, 1.0, 0.0);
		end
	elseif (msg == "setup") then
		CombatCaddy_Main:Show();
	elseif (msg == "status") then
		local status;
		if (CombatCaddy.enabled) then status = "|cff00ff00enabled|r";
		else status = "|cffff0000disabled|r"; end

		DEFAULT_CHAT_FRAME:AddMessage(CombatCaddy.title.." is "..status.."!", 1.0, 1.0, 0.0);
	else
		DEFAULT_CHAT_FRAME:AddMessage("CombatCaddy: unknown command, try /cmbcdy help", 1.0, 1.0, 0.0);
	end
end

-- Handler for switching tabs in the setup screen
CombatCaddy.TabHandler = function(button)
	if (button == "CombatCaddy_MainTab1") then 
		PanelTemplates_SetTab(CombatCaddy_Main, 1);
		CombatCaddy_MainTabPage1:Show();
		CombatCaddy_MainTabPage2:Hide();
		CombatCaddy_MainTabPage3:Hide();
		CombatCaddy_MainTabPage4:Hide();
		CombatCaddy_MainTabPage5:Hide();
	elseif (button == "CombatCaddy_MainTab2") then 
		PanelTemplates_SetTab(CombatCaddy_Main, 2);
		CombatCaddy_MainTabPage1:Hide();
		CombatCaddy_MainTabPage2:Show();
		CombatCaddy_MainTabPage3:Hide();
		CombatCaddy_MainTabPage4:Hide();
		CombatCaddy_MainTabPage5:Hide();
	elseif (button == "CombatCaddy_MainTab3") then 
		PanelTemplates_SetTab(CombatCaddy_Main, 3);
		CombatCaddy_MainTabPage1:Hide();
		CombatCaddy_MainTabPage2:Hide();
		CombatCaddy_MainTabPage3:Show();
		CombatCaddy_MainTabPage4:Hide();
		CombatCaddy_MainTabPage5:Hide();
	elseif (button == "CombatCaddy_MainTab4") then
		PanelTemplates_SetTab(CombatCaddy_Main, 4);
		CombatCaddy_MainTabPage1:Hide();
		CombatCaddy_MainTabPage2:Hide();
		CombatCaddy_MainTabPage3:Hide();
		CombatCaddy_MainTabPage4:Show();
		CombatCaddy_MainTabPage5:Hide();
	elseif (button == "CombatCaddy_MainTab5") then
		PanelTemplates_SetTab(CombatCaddy_Main, 5);
		CombatCaddy_MainTabPage1:Hide();
		CombatCaddy_MainTabPage2:Hide();
		CombatCaddy_MainTabPage3:Hide();
		CombatCaddy_MainTabPage4:Hide();
		CombatCaddy_MainTabPage5:Show();
	end
end

-- Init function for setting up the option check buttons
CombatCaddy.OptionsSetup = function()
	for name, option in pairs(CombatCaddyOptions) do
		local button = getglobal("CombatCaddyOption_"..name);
		if button then
			button.name = name;
			if (option.text) then
				button.text = option.text;
			else
				button.text = "";
			end
		end
	end
end

-- Enables/disables other options when an option is clicked
CombatCaddy.OptionsDeps = function(name)
	if (not WORKING_OPTIONS[name]) then	return; end -- no such option
	if (not WORKING_OPTIONS[name].enables and not WORKING_OPTIONS[name].disables) then return; end -- nothing to do

	-- Iterate through enables list and do appropriate activities
	if (WORKING_OPTIONS[name].enables) then
		local i = 1;
		local cur;
		while true do
			cur = WORKING_OPTIONS[name].enables[i];
			if (not cur) then break; end
			i = i + 1;

			local tmp = getglobal("CombatCaddyOption_" .. cur);
			if (not tmp) then break; end

			if (WORKING_OPTIONS[name].enabled) then
				tmp:Enable();
			else
				tmp:Disable();
			end
		end
	end

	-- Iterate through disables list and do appropriate activities
	if (WORKING_OPTIONS[name].disables) then
		local i = 1;
		local cur;
		while true do
			cur = WORKING_OPTIONS[name].disables[i];
			if (not cur) then break; end
			i = i + 1;

			local tmp = getglobal("CombatCaddyOption_" .. cur);
			if (not tmp) then break; end

			if (WORKING_OPTIONS[name].enabled) then
				tmp:Disable();
			else
				tmp:Enable();
			end
		end
	end
end

-- Automatically determines and sets the checked state and sets the label text for an option
CombatCaddy.CheckButton_OnShow = function()
	this:SetChecked(WORKING_OPTIONS[this.name].enabled);
	getglobal(this:GetName().."Text"):SetText(this.text);
end

-- Sets a field in a CombatCaddy option
-- Expects CombatCaddy.curOptName and CombatCaddy.curOptKey to be set sanely
CombatCaddy.SetOptKey = function(value)
	if ((not CombatCaddy.curOptName) or not (CombatCaddy.curOptKey)) then return; end
	if (not WORKING_OPTIONS[CombatCaddy.curOptName]) then return; end
	if (not WORKING_OPTIONS[CombatCaddy.curOptName][CombatCaddy.curOptKey]) then return; end

	-- Special hooks
	if ((CombatCaddy.curOptName == "CheckLowHealthMinimap") and (CombatCaddy.curOptKey == "thresh")) then
		-- Calculate percentage or use absolute value
		if (strsub(value,strlen(value)) == "%") then
			local tmp = tonumber(strsub(value,1,strlen(value)-1));
			if (not tmp) then return; end -- error catch

			local max = UnitHealthMax("player");
			value = ceil(max * (tmp / 100));
		end
	end

	WORKING_OPTIONS[CombatCaddy.curOptName][CombatCaddy.curOptKey] = value;
	if (type(value) == "table") then
		-- Store a convenient table element count
		WORKING_OPTIONS[CombatCaddy.curOptName][CombatCaddy.curOptKey.."_count"] = #(WORKING_OPTIONS[CombatCaddy.curOptName][CombatCaddy.curOptKey]);
	end
	
	CombatCaddy.curOptName = nil;
	CombatCaddy.curOptKey = nil;
end

-- Goes through the option list and stores the state of the working option list
CombatCaddy.SaveOpts = function()
	for name,_ in pairs(WORKING_OPTIONS) do
		-- Update fields in saved options
		for k,v in pairs(WORKING_OPTIONS[name]) do
			CombatCaddyOptions[name][k] = v;
		end
		-- Remove extra fields in saved options
		for k,v in pairs(CombatCaddyOptions[name]) do
			if (not WORKING_OPTIONS[name][k]) then
				CombatCaddyOptions[name][k] = nil;
			end
		end
	end

	CombatCaddy.Minimap.Update();
end

-- Goes through the working option list and restores it to the state of the saved option list
CombatCaddy.ResetOpts = function()
	for name,_ in pairs(CombatCaddyOptions) do
		-- Update fields in the working options
		for k,v in pairs(CombatCaddyOptions[name]) do
			WORKING_OPTIONS[name][k] = v;
		end
		-- Remove extra fields in working options
		for k,v in pairs(WORKING_OPTIONS[name]) do
			if (not CombatCaddyOptions[name][k]) then
				WORKING_OPTIONS[name][k] = nil;
			end
		end
	end
end

-- Closes the CombatCaddy setup screen
CombatCaddy.CloseSelf = function()
	if (CombatCaddy_Main:IsShown()) then
		CombatCaddy_Main:Hide();
	end
end

-- Hides or shows the mini map
CombatCaddy.HideMinimap = function(hide_mm)
	if (hide_mm == true) then
		CombatCaddy.mmState = MinimapCluster:IsShown();
		MinimapCluster:Hide();
	elseif (CombatCaddy.mmState) then
		MinimapCluster:Show();
	end
end

-- Hides or shows the durability frame
CombatCaddy.HideDurability = function(hide_dur)
	if (hide_dur == true) then
		CombatCaddy.durState = DurabilityFrame:IsShown();
		DurabilityFrame:Hide();
	elseif (CombatCaddy.durState) then
		DurabilityFrame:Show();
	end
end

-- Hides or shows the buff frame
CombatCaddy.HideBuffs = function(hide_bfs)
	if (hide_bfs == true) then
		CombatCaddy.buffState = BuffFrame:IsShown();
		BuffFrame:Hide();
	elseif (CombatCaddy.buffState) then
		BuffFrame:Show();
	end
end

-- Hides or shows the chat frame
CombatCaddy.HideChatFrame = function(hide_mf)
	if (hide_mf == true) then
		CombatCaddy.chatState = SELECTED_CHAT_FRAME:IsShown();
		SELECTED_CHAT_FRAME:Hide();
	elseif (CombatCaddy.chatState) then
		SELECTED_CHAT_FRAME:Show();
	end
end

-- Hides or shows the quest tracker
CombatCaddy.HideQuestTracker = function(hide_qw)
	if (hide_qw == true) then
		CombatCaddy.qwatchState = QuestWatchFrame:IsShown();
		QuestWatchFrame:Hide();
	elseif (CombatCaddy.qwatchState) then
		QuestWatchFrame:Show();
	end
end

-- Closes Achievement frame
CombatCaddy.CloseAchievements = function()
	if (AchievementFrame and AchievementFrame:IsShown()) then
		AchievementFrame:Hide();
	end
end

-- Closes open bags
CombatCaddy.CloseBags = function()
	CloseAllBags();	
end

-- Closes character frame
CombatCaddy.CloseChar = function()
	if (CharacterFrame:IsShown()) then
		CharacterFrame:Hide();
	end
end

-- Closes LFG frame
CombatCaddy.CloseLFG = function()
	if (LFGFrame:IsShown()) then
		LFGFrame:Hide();
	end
end

-- Closes PVP frame
CombatCaddy.ClosePVP = function()
	if (PVPFrame:IsShown()) then
		PVPFrame:Hide();
	end
end

-- Closes the quest log
CombatCaddy.CloseQuestLog = function()
	if (QuestLogFrame:IsShown()) then
		QuestLogFrame:Hide();
	end
end

-- Closes the social frame
CombatCaddy.CloseSocial = function()
	if (FriendsFrame:IsShown()) then
		FriendsFrame:Hide();
	end
end

-- Close the spellbook
CombatCaddy.CloseSpellbook = function()
	if (SpellBookFrame:IsShown()) then
		SpellBookFrame:Hide();
	end
end

-- Closes the world map
CombatCaddy.CloseWorldmap = function()
	if (WorldMapFrame:IsShown()) then
		ToggleFrame(WorldMapFrame);
	end
end

-- Looks for a chat window called Combat Log
CombatCaddy.FindCombatLog = function()
	CombatCaddy.combatLogIdx = nil;

	for i=1, NUM_CHAT_WINDOWS do
		local name,_,_,_,_,_,_,_,_ = GetChatWindowInfo(i);
	
		if (name == CombatCaddyOptions["CheckSwitchToCombatLog"].window) then 
			CombatCaddy.combatLogIdx = i;
			return;
		end
	end
end

-- Switches to combat log
CombatCaddy.SwitchToCombatLog = function(goto_cmblog)
	if (goto_cmblog) then
		CombatCaddy.chatIdx = SELECTED_CHAT_FRAME:GetID();
		CombatCaddy.FindCombatLog();
		if (not CombatCaddy.combatLogIdx) then return; end -- combat log not found
		if (CombatCaddy.combatLogIdx == CombatCaddy.chatIdx) then return; end -- nothing to do

		FCF_SelectDockFrame(getglobal("ChatFrame"..CombatCaddy.combatLogIdx));
	else
		FCF_SelectDockFrame(getglobal("ChatFrame"..CombatCaddy.chatIdx));
	end
end

-- Says a random phrase
CombatCaddy.Battlecry = function()
	local idx = random(1,CombatCaddyOptions["CheckBattlecry"].phrases_count);

	-- Don't say the same thing twice in a row	
	if (CombatCaddyOptions["CheckBattlecry"].lastIdx) then
		if (idx == CombatCaddyOptions["CheckBattlecry"].lastIdx) then
			idx = idx + 1;
		end
		if (idx > CombatCaddyOptions["CheckBattlecry"].phrases_count) then
			idx = 1;
		end
	end

	SendChatMessage(CombatCaddyOptions["CheckBattlecry"].phrases[idx], "SAY");
	CombatCaddyOptions["CheckBattlecry"].lastIdx = idx;
end

-- Plays an mp3
CombatCaddy.PlayMusic = function()
	if (not CombatCaddy.fadeMusic and not CombatCaddy.startMusic) then	
		CombatCaddy.gameMusicVol = tonumber(GetCVar("Sound_MusicVolume"));
	end
	if (not CombatCaddy.gameMusicVol) then 
		return; 
	end

	if (CombatCaddyOptions["CheckUseGameMusicVol"].enabled) then
		CombatCaddy.musicVol = CombatCaddy.gameMusicVol;
	else
		CombatCaddy.musicVol = CombatCaddyOptions["CheckUseGameMusicVol"].volume;
	end

	CombatCaddy.fadeMusic = false;
	CombatCaddy.startMusic = false;

	if (not CombatCaddyOptions["CheckRandomMusic"].enabled) then
		local mp3_path = CombatCaddyOptions["CheckFightMusic"].path .. CombatCaddyOptions["CheckFightMusic"].track;
		PlayMusic(mp3_path);
		SetCVar("Sound_MusicVolume",CombatCaddy.musicVol);
	else
		if (not CombatCaddy.GetNumTracks) then
			DEFAULT_CHAT_FRAME:AddMessage("CombatCaddy: WARNING -- track list not initialized; run gen_tracks.pl to generate track list.", 1.0, 0.0, 0.0);
			return;
		end

		math.randomseed(math.random(0,2147483647)+(GetTime()*1000));
		local idx = math.random(1, CombatCaddy.GetNumTracks());
		local mp3_path = CombatCaddyOptions["CheckFightMusic"].path .. CombatCaddy.TRACKS[idx];

		PlayMusic(mp3_path);
		SetCVar("Sound_MusicVolume",CombatCaddy.musicVol);
	end
end

